/**
 * Build a sitemap.
 *
 * @param {Array<Entry | string> | null | undefined} [data]
 *   Entries to build a sitemap for (optional).
 * @returns {Root}
 *   Sitemap.
 */
export function sitemap(data?: Array<Entry | string> | null | undefined): Root;
/**
 * @param {Date | number | string} value
 * @returns {Date}
 */
export function toDate(value: Date | number | string): Date;
export type Element = import('xast').Element;
export type Root = import('xast').Root;
/**
 * Alternative content, typically a translation.
 *
 * To define different fields, either use a full entry object:
 *
 * ```js
 * [
 * {
 *   url: 'https://example.com/delta/',
 *   modified: '05 October 2011 14:48 UTC',
 *   lang: 'en',
 *   alternate: {nl: {url: 'https://example.com/dirk/', modified: '20 January 2020 00:00 UTC'}}
 * }
 * ]
 * ```
 *
 * …or define them separately:
 *
 * ```js
 * [
 * {
 *   url: 'https://example.com/delta/',
 *   modified: '05 October 2011 14:48 UTC',
 *   lang: 'en',
 *   alternate: {nl: 'https://example.com/dirk/'}
 * },
 * {
 *   url: 'https://example.com/dirk/',
 *   modified: '20 January 2020 00:00 UTC',
 *   // `xast-util-sitemap` is smart enough to know about the next two already,
 *   // but they’re shown here for clarity.
 *   lang: 'nl',
 *   alternate: {en: 'https://example.com/delta/'}
 * }
 * ]
 * ```
 */
export type Alternate = Omit<Entry, 'alternate' | 'lang'> | string;
/**
 * Entries represent a single URL and describe them with metadata.
 */
export type Entry = {
    /**
     *   Full URL (`<loc>`, example: `'https://example.org/'`).
     */
    url: string;
    /**
     * Value indicating when the page last changed (`<lastmod>`) (optional).
     */
    modified?: Date | number | string | null | undefined;
    /**
     * BCP 47 tag indicating the language of the page (example: `'en-GB'`,
     * optional).
     */
    lang?: string | null | undefined;
    /**
     * Translations of the page, where each key is a BCP 47 tag and each value an
     * entry (example: `{nl: 'https://example.nl/'}`, optional).
     *
     * Alternate resources inherit fields from the entry they are described in.
     */
    alternate?: Record<string, Alternate> | null | undefined;
};
